# coding=utf-8
"""Actividad 2"""

import glfw
from OpenGL.GL import (glUseProgram, glClearColor, glEnable, GL_DEPTH_TEST,
                       GL_STATIC_DRAW, glUniformMatrix4fv, glGetUniformLocation,
                       GL_TRUE, glClear, GL_COLOR_BUFFER_BIT, GL_DEPTH_BUFFER_BIT,
                       glPolygonMode, GL_FRONT_AND_BACK, GL_FILL, GL_LINES)
import numpy as np
import grafica.transformations as tr
import grafica.basic_shapes as bs
import grafica.easy_shaders as es

#Funciones de las curvas

def generateT(t):
    return np.array([[1, t, t**2, t**3]]).T

def hermiteMatrix(P1, P2, T1, T2):
    
    # Generate a matrix concatenating the columns
    G = np.concatenate((P1, P2, T1, T2), axis=1)
    
    # Hermite base matrix is a constant
    Mh = np.array([[1, 0, -3, 2], [0, 0, 3, -2], [0, 1, -2, 1], [0, 0, -1, 1]])    
    
    return np.matmul(G, Mh)

    
def bezierMatrix(P0, P1, P2, P3):
    
    # Generate a matrix concatenating the columns
    G = np.concatenate((P0, P1, P2, P3), axis=1)

    # Bezier base matrix is a constant
    Mb = np.array([[1, -3, 3, -1], [0, 3, -6, 3], [0, 0, 3, -3], [0, 0, 0, 1]])
    
    return np.matmul(G, Mb)



# M is the cubic curve matrix, N is the number of samples between 0 and 1
def evalCurve(M, N):
    # The parameter t should move between 0 and 1
    ts = np.linspace(0.0, 1.0, N)
    
    # The computed value in R3 for each sample will be stored here
    curve = np.ndarray(shape=(N, 3), dtype=float)
    
    for i in range(len(ts)):
        T = generateT(ts[i])
        curve[i, 0:3] = np.matmul(M, T).T
        
    return curve





def main():

    # Initialize glfw
    if not glfw.init():
        glfw.set_window_should_close(window, True)

    width = 600
    height = 600

    window = glfw.create_window(width, height, "Projections Demo", None, None)

    if not window:
        glfw.terminate()
        glfw.set_window_should_close(window, True)

    glfw.make_context_current(window)

    # Assembling the shader program
    pipeline = es.SimpleModelViewProjectionShaderProgram()

    # Telling OpenGL to use our shader program
    glUseProgram(pipeline.shaderProgram)

    # Setting up the clear screen color
    glClearColor(0.15, 0.15, 0.15, 1.0)

    # As we work in 3D, we need to check which part is in front,
    # and which one is at the back
    glEnable(GL_DEPTH_TEST)

    # Convenience function to ease initialization
    def createGPUShape(shape):
        gpuShape = es.GPUShape().initBuffers()
        pipeline.setupVAO(gpuShape)
        gpuShape.fillBuffers(shape.vertices, shape.indices, GL_STATIC_DRAW)
        return gpuShape

    # Creating shapes on GPU memory
    gpuAxis = createGPUShape(bs.createAxis(7))
    gpuRedCube = createGPUShape(bs.createColorCube(1, 0, 0))
    gpuGreenCube = createGPUShape(bs.createColorCube(0, 1, 0))

    camera_theta = np.pi/4

    # Setting up the view transform

    cam_radius = 10
    cam_x = cam_radius * np.sin(camera_theta)
    cam_y = cam_radius * np.cos(camera_theta)
    cam_z = cam_radius

    viewPos = np.array([cam_x, cam_y, cam_z])

    view = tr.lookAt(viewPos, np.array([0, 0, 0]), np.array([0, 0, 1]))

    # Setting up the projection transform

    projection = tr.ortho(-8, 8, -8, 8, 0.1, 100)


    #Cramos las curvas

    #Definimos una variable para tener el
    #numero de iteraciones que consideraremos
    N = 3000


    #Creando una curva de Hermite
    #Definimos los puntos
    P1 = np.array([[0, 0, -5]]).T
    P2 = np.array([[0, 0, 5]]).T
    T1 = np.array([[60, 0, 0]]).T
    T2 = np.array([[60, 0, 0]]).T

    #Creamos la curva
    GMh = hermiteMatrix(P1, P2, T1, T2)
    HermiteCurve = evalCurve(GMh, N)



    #Creando las curvas de Bezier
    
    #Definimos los puntos de una primera curva
    P0 = np.array([[0, 0, -5]]).T
    P1 = np.array([[-10, 0, -5]]).T
    P2 = np.array([[-10, 0, 0]]).T
    P3 = np.array([[0, 0, 0]]).T
    
    #Creamos la primera parte de la curva
    M1 = bezierMatrix(P0, P1, P2, P3)
    bezierCurve1 = evalCurve(M1, N//2)

    #Definimos los puntos de una segunda curva
    P4 = np.array([[0, 0, 0]]).T
    P5 = np.array([[10, 0, 0]]).T
    P6 = np.array([[10, 0, 5]]).T
    P7 = np.array([[0, 0, 5]]).T
    
    #Creamos la segunda parte de la curva
    M2 = bezierMatrix(P4,P5,P6,P7)
    bezierCurve2 = evalCurve(M2, N//2)

    #Se pueden concatenar las 2 curvas de Bezier en una sola
    BezierCurve = np.concatenate((bezierCurve1,bezierCurve2), axis=0)


    step = 0
    
    ##### UN CONTADOR QUE NOS ASISTIRA EN LAS TRANSFORMACIONES #####
    t0 = glfw.get_time()

    while not glfw.window_should_close(window):

        # Using GLFW to check for input events
        glfw.poll_events()

        ##### CALCULAMOS EL DELTA DEL TIEMPO #####

        t1 = glfw.get_time()
        dt = t1 - t0
        t0 = t1

        #Agregamos los step para tener la cuena de las iteraciones
        #Si step es mayor a N, reseteamos su cuenta
        if step >= N-1:
            step = 0

        step = step + 1

        ##### ENTRAREMOS EN MAS DETALLE SOBRE VIEW Y PROJECTION EN EL FUTURO #####

        glUniformMatrix4fv(glGetUniformLocation(pipeline.shaderProgram, "view"), 1, GL_TRUE, view)

        glUniformMatrix4fv(glGetUniformLocation(pipeline.shaderProgram, "projection"), 1, GL_TRUE, projection)

        # Clearing the screen in both, color and depth
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

        glPolygonMode(GL_FRONT_AND_BACK, GL_FILL)

        ##### DEBEN MODIFICAR ESTAS LINEAS PARA EL AUXILIAR 2 #####



        #En cada iteración actualizamos la posición del cubo rojo
        #Llamamos a la curva de Hermite definida
        transformRed = tr.matmul([
            tr.translate(HermiteCurve[step,0], HermiteCurve[step,1], HermiteCurve[step,2])
        ])
        glUniformMatrix4fv(glGetUniformLocation(pipeline.shaderProgram, "model"), 1, GL_TRUE, transformRed)
        pipeline.drawCall(gpuRedCube)


        #En cada iteración actualizamos la posición del cubo verde
        #Llamamos a la curva de Bezier definida
        transformGreen = tr.matmul([
            tr.translate(BezierCurve[step,0], BezierCurve[step,1], BezierCurve[step,2])
        ])
        glUniformMatrix4fv(glGetUniformLocation(pipeline.shaderProgram, "model"), 1, GL_TRUE, transformGreen)
        pipeline.drawCall(gpuGreenCube)

        glUniformMatrix4fv(glGetUniformLocation(pipeline.shaderProgram, "model"), 1, GL_TRUE, tr.identity())
        pipeline.drawCall(gpuAxis, GL_LINES)

        ##### NO ES NECESARIO QUE MODIFIQUEN NADA DESPUES DE ESTE PUNTO #####

        # Once the drawing is rendered, buffers are swap so an uncomplete drawing is never seen.
        glfw.swap_buffers(window)

    # freeing GPU memory
    gpuRedCube.clear()
    gpuGreenCube.clear()

    glfw.terminate()


if __name__ == "__main__":
    main()
